% Copyright (c) 2023, Lucia Maddalena
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
% 
% * The above copyright notice and this permission notice shall be included in all
%   copies or substantial portions of the Software.
% * In case results obtained with the present software, or parts of it, are
%   published in a scientific paper, the following reference should be cited:
%
%   L. Maddalena, I. Granata, M. Giordano, M. Manzo, M.R. Guarracino
%   Integrating Different Data Modalities for the Classification of
%   Alzheimer’s Disease Stages, 
%   DOI: 10.1007/s42979-023-01688-2,
%   SN Computer Science, vol. 4, no. 249, 2023
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Given:
%- CR = computed result
%- GT = true result
%computes the metrics M = [Acc, Sens, Spec, Prec, F-measure, G-mean, AUC, MCC]
%related to class "ForClass"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function M = ClassificationMetrics(GT, CR, Score, ForClass)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
M = zeros(1,8);

TP = sum(CR==ForClass & GT==ForClass);
FP = sum(CR==ForClass & GT~=ForClass);
TN = sum(CR~=ForClass & GT~=ForClass);
FN = sum(CR~=ForClass & GT==ForClass);
n = length(GT);
%Acc:
M(1) = (TP+TN)/n;
%Sens:
if (TP+FN)
    M(2) = TP/(TP+FN);
end
%Spec:
if (TN+FP)
    M(3) = TN/(TN+FP);
end
%Prec:
if (TP+FP)
    M(4) = TP/(TP+FP);
end
%F-measure:
if (M(4)+M(2))
    M(5) = 2*M(4)*M(2)/(M(4)+M(2));
end
%G-mean:
M(6) = sqrt(M(2)*M(3));
%AUC:
if (length(unique(GT))>1)
    [X,Y,T,AUC] = perfcurve(GT,Score(:,1),ForClass);
else
    AUC=1; fprintf('*** ATTENTION: ONLY 1 CLASS IN THE TESTING SET ***\n')
end
M(7) = AUC;
%MCC
P=TP+FN;
N=FP+TN;
den = (TP+FP).*P.*N.*(TN+FN);
if (den)
    M(8) = ( TP.*TN - FP.*FN ) ./ ( den.^(0.5) );
else
    M(8) = 0;
end
end
