% Copyright (c) 2023, Lucia Maddalena
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
% 
% * The above copyright notice and this permission notice shall be included in all
%   copies or substantial portions of the Software.
% * In case results obtained with the present software, or parts of it, are
%   published in a scientific paper, the following reference should be cited:
%
%   L. Maddalena, I. Granata, M. Giordano, M. Manzo, M.R. Guarracino
%   Integrating Different Data Modalities for the Classification of
%   Alzheimer’s Disease Stages, 
%   DOI: 10.1007/s42979-023-01688-2,
%   SN Computer Science, vol. 4, no. 249,  2023
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Evaluation procedure:
%It consists of NumIter iterations of k-fold cross-validation, with 
%stratified partitions of the data into train and test subsets, 
%using different classifiers.
%At each iteration, data are standardized by z-scoring training folds and 
%using their mean and variance to z-score the test set accordingly.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [AverageMetrics] = EvaluationProcedure(Tall,LabelClassPos,LabelClassNeg,...
                                                NumIterPart,NumFolds,ClassType)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Select only the subset of data belonging to the two specified classes
Sel  = strcmp(Tall.Diagnosis,LabelClassPos)|strcmp(Tall.Diagnosis,LabelClassNeg);
T    = Tall(Sel,:);
T    = sortrows(T,size(T,2));
Data = T{:,2:end-1};

%Compute numerical labels
Label = strcmp(T.Diagnosis,LabelClassPos);

%Class 1 assumed as positive class
ForClass =1;

Mfolds   = zeros(NumFolds,8);    %Performance metrics for each fold
FoldMean = zeros(NumIterPart,8); %Average performance metrics for each iteration

for k=1:NumIterPart
    rng(k); %Enforce repeatable results
    %Randomly partition observations into a training and a test set using stratified Kfold
    P = cvpartition(Label,'KFold',NumFolds,'Stratify',true);
    for i = 1:NumFolds
        %a) Training
        train = P.training(i);
        test = P.test(i);
        
        %z-score training data
        [DataTrain, mu, sigma] = zscore(Data(train,:));
        %z-score test data, according to mean and std of training data
        DataTest = (Data(test,:) - mu)./sigma;

        %Classification model construction
        rng(i); %Enforce repeatable results
        [Mdl] = funModelFit(ClassType, DataTrain, Label(train));

        %b) Testing
        rng(i); %Enforce repeatable results
        %Prediction on the test data
        [Pred, Score] = predict(Mdl, DataTest);
        %Computation of the performance metrics (ForClass assumed as positive class)
        ScoreForClass = Score(:,find(Mdl.ClassNames==ForClass));
        Mfolds(i,:)   = ClassificationMetrics(Label(test), Pred, ScoreForClass, ForClass);    
    end
    FoldMean(k,:) = mean(Mfolds,1);%Mean over all folds
end
%Adds Balanced Accuracy to the other metrics
BA = mean([mean(FoldMean(:,2),1),  mean(FoldMean(:,3),1)]);
AverageMetrics = [mean(FoldMean(:,:),1),BA];

