% Copyright (c) 2023, Lucia Maddalena
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
% 
% * The above copyright notice and this permission notice shall be included in all
%   copies or substantial portions of the Software.
% * In case results obtained with the present software, or parts of it, are
%   published in a scientific paper, the following reference should be cited:
%
%   L. Maddalena, I. Granata, M. Giordano, M. Manzo, M.R. Guarracino
%   Integrating Different Data Modalities for the Classification of
%   Alzheimer’s Disease Stages, 
%   DOI: 10.1007/s42979-023-01688-2,
%   SN Computer Science, vol. 4, no. 249, 2023
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Main example program to evaluate classification on the ANMerge dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%The best combinations of 
% 1) classifier
% 2) Atlas for MRI data
% 3) Method for GE data
%are chosen for each binary classification task (AD vs CN, AD vs MCI, MCI vs CN) 
%and for each set of data (MRI, GE, MRI+GE), to reproduce Table 3 of the paper.
%
clear all

%Folders for input data:
DirMRIfeats = './ANMfeats/ANM_MRI';
DirGEfeats  = './ANMfeats/ANM_GE';

%Fixed values for evaluation parameters
NumFolds    = 5;
NumIterPart = 50;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Binary problem to be solved: AD vs CN
LabelClassPos='AD'; LabelClassNeg='CTL'; 
fprintf('%%%%%%%%%%%%%% %s vs %s %%%%%%%%%%%%%%\n',LabelClassPos,LabelClassNeg);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%MRI
ClassType = 8;        % 1) classifier: LR
Atlas     = 'AAL2';   % 2) Atlas for MRI data: AAL2
fprintf('MRI:    Classifier %d, Atlas %s\n',ClassType,Atlas);

FileNameMRI = sprintf('%s/%s%s_%s.csv',DirMRIfeats,LabelClassPos,LabelClassNeg,Atlas);
T_MRI = readtable(FileNameMRI,'PreserveVariableNames',true);
T_MRI = sortrows(T_MRI);
T_MRI.Properties.VariableNames(end) = {'Diagnosis'};

[AvgMetrics_MRI] = EvaluationProcedure(T_MRI,LabelClassPos,LabelClassNeg,...
                                       NumIterPart,NumFolds,ClassType);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%GE
ClassType = 1;             % 1) classifier: SVM with linear kernel
GEmethod  = 'ADCTL_pv005'; % 3) Method for GE data: "AD-CN_pv005"
NameGE    = sprintf('%s/%s.csv',DirGEfeats,GEmethod);
fprintf('GE:     Classifier %d, GE method %s\n',ClassType,GEmethod);

T_GE = readtable(NameGE,'Delimiter',',','PreserveVariableNames',true);
T_GE = sortrows(T_GE);
T_GE.Properties.VariableNames(end)={'Diagnosis'};

[AvgMetrics_GE] = EvaluationProcedure(T_GE,LabelClassPos,LabelClassNeg,...
                                      NumIterPart,NumFolds,ClassType);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%MRI+GE
ClassType = 1;             % 1) classifier: SVM with linear kernel
Atlas     = 'AAL2';        % 2) Atlas for MRI data: AAL2
GEmethod  = 'ADCTL_pv005'; % 3) Method for GE data: "AD-CN_pv005"
fprintf('MRI+GE: Classifier %d, Atlas %s, GE method %s\n',ClassType,Atlas,GEmethod);

%T_MRI and T_GE with best choices already read
T_MRI_GE = [T_MRI(:,1:end-1), T_GE(:,2:end)];
[AvgMetrics_MRI_GE] = EvaluationProcedure(T_MRI_GE,LabelClassPos,LabelClassNeg,...
                                          NumIterPart,NumFolds,ClassType);
fprintf('Results:\n');
fprintf('        Acc    Sens   Spec   Prec   F1     Gm     AUC    MCC    BA\n');
fprintf('MRI     %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f\n', AvgMetrics_MRI);
fprintf('GE      %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f\n', AvgMetrics_GE);
fprintf('MRI+GE  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f\n', AvgMetrics_MRI_GE);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Binary problem to be solved: AD vs MCI
LabelClassPos='AD'; LabelClassNeg='MCI'; 
fprintf('%%%%%%%%%%%%%% %s vs %s %%%%%%%%%%%%%%\n',LabelClassPos,LabelClassNeg);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%MRI
ClassType = 8;        % 1) classifier: LR
Atlas     = 'AAL2';   % 2) Atlas for MRI data: AAL2
fprintf('MRI:    Classifier %d, Atlas %s\n',ClassType,Atlas);

FileNameMRI = sprintf('%s/%s%s_%s.csv',DirMRIfeats,LabelClassPos,LabelClassNeg,Atlas);
T_MRI = readtable(FileNameMRI,'PreserveVariableNames',true);
T_MRI = sortrows(T_MRI);
T_MRI.Properties.VariableNames(end) = {'Diagnosis'};

[AvgMetrics_MRI] = EvaluationProcedure(T_MRI,LabelClassPos,LabelClassNeg,...
                                           NumIterPart,NumFolds,ClassType);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%GE
ClassType = 1;             % 1) classifier: SVM with linear kernel
GEmethod  = 'ADMCI_pv005'; % 3) Method for GE data: "AD-CN_pv005"
fprintf('GE:     Classifier %d, GE method %s\n',ClassType,GEmethod);

NameGE = sprintf('%s/%s.csv',DirGEfeats,GEmethod);
T_GE = readtable(NameGE,'Delimiter',',','PreserveVariableNames',true);
T_GE = sortrows(T_GE);
T_GE.Properties.VariableNames(end)={'Diagnosis'};

[AvgMetrics_GE] = EvaluationProcedure(T_GE,LabelClassPos,LabelClassNeg,...
                                          NumIterPart,NumFolds,ClassType);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%MRI+GE
ClassType = 8;             % 1) classifier: LR
Atlas     = 'AAL2';        % 2) Atlas for MRI data: AAL2
GEmethod  = 'ADMCI_pv005'; % 3) Method for GE data: "AD-MCI_pv005"
fprintf('MRI+GE: Classifier %d, Atlas %s, GE method %s\n',ClassType,Atlas,GEmethod);

FileNameMRI = sprintf('%s/%s%s_%s.csv',DirMRIfeats,LabelClassPos,LabelClassNeg,Atlas);
T_MRI = readtable(FileNameMRI,'PreserveVariableNames',true);
T_MRI = sortrows(T_MRI);
T_MRI.Properties.VariableNames(end) = {'Diagnosis'};

%T_GE with best choices already read
T_MRI_GE = [T_MRI(:,1:end-1), T_GE(:,2:end)];
[AvgMetrics_MRI_GE] = EvaluationProcedure(T_MRI_GE,LabelClassPos,LabelClassNeg,...
                                              NumIterPart,NumFolds,ClassType);
fprintf('Results:\n');
fprintf('        Acc    Sens   Spec   Prec   F1     Gm     AUC    MCC    BA\n');
fprintf('MRI     %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f\n', AvgMetrics_MRI);
fprintf('GE      %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f\n', AvgMetrics_GE);
fprintf('MRI+GE  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f\n', AvgMetrics_MRI_GE);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Binary problem to be solved: MCI vs CN
LabelClassPos='MCI'; LabelClassNeg='CTL'; 
fprintf('%%%%%%%%%%%%%% %s vs %s %%%%%%%%%%%%%%\n',LabelClassPos,LabelClassNeg);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%MRI
ClassType = 8;      % 1) classifier: LR
Atlas     = 'AAL2'; % 2) Atlas for MRI data: AAL2
fprintf('MRI:    Classifier %d, Atlas %s\n',ClassType,Atlas);

FileNameMRI = sprintf('%s/%s%s_%s.csv',DirMRIfeats,LabelClassPos,LabelClassNeg,Atlas);
T_MRI = readtable(FileNameMRI,'PreserveVariableNames',true);
T_MRI = sortrows(T_MRI);
T_MRI.Properties.VariableNames(end) = {'Diagnosis'};

[AvgMetrics_MRI] = EvaluationProcedure(T_MRI,LabelClassPos,LabelClassNeg,...
                                           NumIterPart,NumFolds,ClassType);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%GE
ClassType = 1;                    % 1) classifier: SVM with linear kernel
GEmethod  = 'MCICTL_pv005_LFC03'; % 3) Method for GE data: "'MCI-CN_pv005_LFC03';"
fprintf('GE:     Classifier %d, GE method %s\n',ClassType,GEmethod);

NameGE = sprintf('%s/%s.csv',DirGEfeats,GEmethod);
T_GE = readtable(NameGE,'Delimiter',',','PreserveVariableNames',true);
T_GE = sortrows(T_GE);
T_GE.Properties.VariableNames(end)={'Diagnosis'};

[AvgMetrics_GE] = EvaluationProcedure(T_GE,LabelClassPos,LabelClassNeg,...
                                          NumIterPart,NumFolds,ClassType);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%MRI+GE
ClassType = 1;                    % 1) classifier: SVM with linear kernel
Atlas     = 'AAL2';               % 2) Atlas for MRI data: AAL2
GEmethod  = 'MCICTL_pv005_LFC03'; % 3) Method for GE data: "'MCI-CN_pv005_LFC03';"
fprintf('MRI+GE: Classifier %d, Atlas %s, GE method %s\n',ClassType,Atlas,GEmethod);

FileNameMRI = sprintf('%s/%s%s_%s.csv',DirMRIfeats,LabelClassPos,LabelClassNeg,Atlas);
T_MRI = readtable(FileNameMRI,'PreserveVariableNames',true);
T_MRI = sortrows(T_MRI);
T_MRI.Properties.VariableNames(end) = {'Diagnosis'};

%T_GE with best choices already read
T_MRI_GE = [T_MRI(:,1:end-1), T_GE(:,2:end)];
[AvgMetrics_MRI_GE] = EvaluationProcedure(T_MRI_GE,LabelClassPos,LabelClassNeg,...
                                              NumIterPart,NumFolds,ClassType);
fprintf('Results:\n');
fprintf('        Acc    Sens   Spec   Prec   F1     Gm     AUC    MCC    BA\n');
fprintf('MRI     %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f\n', AvgMetrics_MRI);
fprintf('GE      %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f\n', AvgMetrics_GE);
fprintf('MRI+GE  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f  %.3f\n', AvgMetrics_MRI_GE);
