% Copyright (c) 2023, Lucia Maddalena
% 
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
% 
% * The above copyright notice and this permission notice shall be included in all
%   copies or substantial portions of the Software.
% * In case results obtained with the present software, or parts of it, are
%   published in a scientific paper, the following reference should be cited:
%
%   L. Maddalena, I. Granata, M. Giordano, M. Manzo, M.R. Guarracino
%   Integrating Different Data Modalities for the Classification of
%   Alzheimer’s Disease Stages, 
%   DOI: 10.1007/s42979-023-01688-2,
%   SN Computer Science, vol. 4, no. 249, 2023
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Mdl] = funModelFit(ClassType, X_train, Y_train)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch(ClassType)
    case 1 %SVM with linear kernel
        Mdl = fitcsvm(X_train,Y_train,'ScoreTransform','logit'); 
    case 2 %SVM with polynomial kernel
        Mdl = fitcsvm(X_train,Y_train,'KernelFunction','polynomial','PolynomialOrder',3,'ScoreTransform','logit'); 
    case 3 %SVM with Gaussian kernel
        Mdl = fitcsvm(X_train,Y_train,'KernelFunction','gaussian','ScoreTransform','logit'); 
    case 4 %Naive Bayes
        Mdl = fitcnb(X_train,Y_train,'ScoreTransform','logit'); 
    case 5 %kNN
        Mdl = fitcknn(X_train,Y_train,'ScoreTransform','logit'); 
    case 6 %tree
        Mdl = fitctree(X_train,Y_train,'ScoreTransform','logit'); 
    case 7 %ensemble
        Mdl = fitcensemble(X_train,Y_train,'ScoreTransform','logit');
    case 8 %fitclinear
        Mdl = funFitcLinear(X_train,Y_train);
    case 9 %RUSboost
        t = templateTree('MaxNumSplits',size(X_train,1));
        Mdl = fitcensemble(X_train,Y_train,'Method','RUSBoost', ...
                'NumLearningCycles',1000,'Learners',t,'LearnRate',0.1); 
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [MdlFinal] = funFitcLinear(data, numLabels)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%See https://it.mathworks.com/help/stats/fitclinear.html
%
%Create a set of 11 logarithmically-spaced regularization strengths
Lambda = logspace(-6,-0.5,11);

%Cross-validate the models 
X = data'; 
CVMdl = fitclinear(X,numLabels,'ObservationsIn','columns','KFold',5,...
             'Learner','logistic','Regularization','lasso','Lambda',Lambda);

%Estimate the cross-validated classification error
ce = kfoldLoss(CVMdl);

%Determine the number of nonzero coefficients per model
Mdl = fitclinear(X,numLabels,'ObservationsIn','columns',...
            'Learner','logistic','Regularization','lasso','Lambda',Lambda); 
numNZCoeff = sum(Mdl.Beta~=0);

%Choose the index of the regularization strength that balances predictor 
%variable sparsity and low classification error
[~,idxFinal]=min(log10(ce).*log10(numNZCoeff));

%Select the model from Mdl with the chosen regularization strength
MdlFinal = selectModels(Mdl,idxFinal);
end